# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hedgehog/protocol/proto/speaker.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hedgehog/protocol/proto/speaker.proto',
  package='hedgehog.protocol.proto',
  syntax='proto3',
  serialized_pb=_b('\n%hedgehog/protocol/proto/speaker.proto\x12\x17hedgehog.protocol.proto\"\"\n\rSpeakerAction\x12\x11\n\tfrequency\x18\x01 \x01(\rB+\n\x1f\x61t.pria.hedgehog.protocol.protoB\x08SpeakerPb\x06proto3')
)




_SPEAKERACTION = _descriptor.Descriptor(
  name='SpeakerAction',
  full_name='hedgehog.protocol.proto.SpeakerAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='frequency', full_name='hedgehog.protocol.proto.SpeakerAction.frequency', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=100,
)

DESCRIPTOR.message_types_by_name['SpeakerAction'] = _SPEAKERACTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SpeakerAction = _reflection.GeneratedProtocolMessageType('SpeakerAction', (_message.Message,), dict(
  DESCRIPTOR = _SPEAKERACTION,
  __module__ = 'hedgehog.protocol.proto.speaker_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.SpeakerAction)
  ))
_sym_db.RegisterMessage(SpeakerAction)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\037at.pria.hedgehog.protocol.protoB\010SpeakerP'))
# @@protoc_insertion_point(module_scope)
