# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hedgehog/protocol/proto/imu.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hedgehog.protocol.proto import subscription_pb2 as hedgehog_dot_protocol_dot_proto_dot_subscription__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hedgehog/protocol/proto/imu.proto',
  package='hedgehog.protocol.proto',
  syntax='proto3',
  serialized_pb=_b('\n!hedgehog/protocol/proto/imu.proto\x12\x17hedgehog.protocol.proto\x1a*hedgehog/protocol/proto/subscription.proto\"\x9a\x01\n\nImuMessage\x12.\n\x04kind\x18\x01 \x01(\x0e\x32 .hedgehog.protocol.proto.ImuKind\x12\t\n\x01x\x18\x02 \x01(\x11\x12\t\n\x01y\x18\x03 \x01(\x11\x12\t\n\x01z\x18\x04 \x01(\x11\x12;\n\x0csubscription\x18\x05 \x01(\x0b\x32%.hedgehog.protocol.proto.Subscription*/\n\x07ImuKind\x12\x08\n\x04RATE\x10\x00\x12\x10\n\x0c\x41\x43\x43\x45LERATION\x10\x01\x12\x08\n\x04POSE\x10\x02\x42\'\n\x1f\x61t.pria.hedgehog.protocol.protoB\x04ImuPb\x06proto3')
  ,
  dependencies=[hedgehog_dot_protocol_dot_proto_dot_subscription__pb2.DESCRIPTOR,])

_IMUKIND = _descriptor.EnumDescriptor(
  name='ImuKind',
  full_name='hedgehog.protocol.proto.ImuKind',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RATE', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ACCELERATION', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='POSE', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=263,
  serialized_end=310,
)
_sym_db.RegisterEnumDescriptor(_IMUKIND)

ImuKind = enum_type_wrapper.EnumTypeWrapper(_IMUKIND)
RATE = 0
ACCELERATION = 1
POSE = 2



_IMUMESSAGE = _descriptor.Descriptor(
  name='ImuMessage',
  full_name='hedgehog.protocol.proto.ImuMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='kind', full_name='hedgehog.protocol.proto.ImuMessage.kind', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='x', full_name='hedgehog.protocol.proto.ImuMessage.x', index=1,
      number=2, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='y', full_name='hedgehog.protocol.proto.ImuMessage.y', index=2,
      number=3, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='z', full_name='hedgehog.protocol.proto.ImuMessage.z', index=3,
      number=4, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='subscription', full_name='hedgehog.protocol.proto.ImuMessage.subscription', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=107,
  serialized_end=261,
)

_IMUMESSAGE.fields_by_name['kind'].enum_type = _IMUKIND
_IMUMESSAGE.fields_by_name['subscription'].message_type = hedgehog_dot_protocol_dot_proto_dot_subscription__pb2._SUBSCRIPTION
DESCRIPTOR.message_types_by_name['ImuMessage'] = _IMUMESSAGE
DESCRIPTOR.enum_types_by_name['ImuKind'] = _IMUKIND
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ImuMessage = _reflection.GeneratedProtocolMessageType('ImuMessage', (_message.Message,), dict(
  DESCRIPTOR = _IMUMESSAGE,
  __module__ = 'hedgehog.protocol.proto.imu_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.ImuMessage)
  ))
_sym_db.RegisterMessage(ImuMessage)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\037at.pria.hedgehog.protocol.protoB\004ImuP'))
# @@protoc_insertion_point(module_scope)
