# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hedgehog/protocol/proto/version.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hedgehog/protocol/proto/version.proto',
  package='hedgehog.protocol.proto',
  syntax='proto3',
  serialized_pb=_b('\n%hedgehog/protocol/proto/version.proto\x12\x17hedgehog.protocol.proto\"k\n\x0eVersionMessage\x12\r\n\x05uc_id\x18\x01 \x01(\x0c\x12\x18\n\x10hardware_version\x18\x02 \x01(\t\x12\x18\n\x10\x66irmware_version\x18\x03 \x01(\t\x12\x16\n\x0eserver_version\x18\x04 \x01(\tB+\n\x1f\x61t.pria.hedgehog.protocol.protoB\x08VersionPb\x06proto3')
)




_VERSIONMESSAGE = _descriptor.Descriptor(
  name='VersionMessage',
  full_name='hedgehog.protocol.proto.VersionMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uc_id', full_name='hedgehog.protocol.proto.VersionMessage.uc_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hardware_version', full_name='hedgehog.protocol.proto.VersionMessage.hardware_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='firmware_version', full_name='hedgehog.protocol.proto.VersionMessage.firmware_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='server_version', full_name='hedgehog.protocol.proto.VersionMessage.server_version', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=173,
)

DESCRIPTOR.message_types_by_name['VersionMessage'] = _VERSIONMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VersionMessage = _reflection.GeneratedProtocolMessageType('VersionMessage', (_message.Message,), dict(
  DESCRIPTOR = _VERSIONMESSAGE,
  __module__ = 'hedgehog.protocol.proto.version_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.VersionMessage)
  ))
_sym_db.RegisterMessage(VersionMessage)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\037at.pria.hedgehog.protocol.protoB\010VersionP'))
# @@protoc_insertion_point(module_scope)
