# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hedgehog/protocol/proto/emergency.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hedgehog.protocol.proto import subscription_pb2 as hedgehog_dot_protocol_dot_proto_dot_subscription__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hedgehog/protocol/proto/emergency.proto',
  package='hedgehog.protocol.proto',
  syntax='proto3',
  serialized_pb=_b('\n\'hedgehog/protocol/proto/emergency.proto\x12\x17hedgehog.protocol.proto\x1a*hedgehog/protocol/proto/subscription.proto\"#\n\x0f\x45mergencyAction\x12\x10\n\x08\x61\x63tivate\x18\x01 \x01(\x08\"_\n\x10\x45mergencyMessage\x12\x0e\n\x06\x61\x63tive\x18\x01 \x01(\x08\x12;\n\x0csubscription\x18\x02 \x01(\x0b\x32%.hedgehog.protocol.proto.SubscriptionB-\n\x1f\x61t.pria.hedgehog.protocol.protoB\nEmergencyPb\x06proto3')
  ,
  dependencies=[hedgehog_dot_protocol_dot_proto_dot_subscription__pb2.DESCRIPTOR,])




_EMERGENCYACTION = _descriptor.Descriptor(
  name='EmergencyAction',
  full_name='hedgehog.protocol.proto.EmergencyAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='activate', full_name='hedgehog.protocol.proto.EmergencyAction.activate', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=112,
  serialized_end=147,
)


_EMERGENCYMESSAGE = _descriptor.Descriptor(
  name='EmergencyMessage',
  full_name='hedgehog.protocol.proto.EmergencyMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='active', full_name='hedgehog.protocol.proto.EmergencyMessage.active', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='subscription', full_name='hedgehog.protocol.proto.EmergencyMessage.subscription', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=149,
  serialized_end=244,
)

_EMERGENCYMESSAGE.fields_by_name['subscription'].message_type = hedgehog_dot_protocol_dot_proto_dot_subscription__pb2._SUBSCRIPTION
DESCRIPTOR.message_types_by_name['EmergencyAction'] = _EMERGENCYACTION
DESCRIPTOR.message_types_by_name['EmergencyMessage'] = _EMERGENCYMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EmergencyAction = _reflection.GeneratedProtocolMessageType('EmergencyAction', (_message.Message,), dict(
  DESCRIPTOR = _EMERGENCYACTION,
  __module__ = 'hedgehog.protocol.proto.emergency_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.EmergencyAction)
  ))
_sym_db.RegisterMessage(EmergencyAction)

EmergencyMessage = _reflection.GeneratedProtocolMessageType('EmergencyMessage', (_message.Message,), dict(
  DESCRIPTOR = _EMERGENCYMESSAGE,
  __module__ = 'hedgehog.protocol.proto.emergency_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.EmergencyMessage)
  ))
_sym_db.RegisterMessage(EmergencyMessage)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\037at.pria.hedgehog.protocol.protoB\nEmergencyP'))
# @@protoc_insertion_point(module_scope)
