# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hedgehog/protocol/proto/vision.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hedgehog/protocol/proto/vision.proto',
  package='hedgehog.protocol.proto',
  syntax='proto3',
  serialized_pb=_b('\n$hedgehog/protocol/proto/vision.proto\x12\x17hedgehog.protocol.proto\"\x0e\n\x0c\x46\x61\x63\x65sChannel\"0\n\x0c\x42lobsChannel\x12\x0f\n\x07hsv_min\x18\x01 \x01(\x07\x12\x0f\n\x07hsv_max\x18\x02 \x01(\x07\"\x91\x01\n\x07\x43hannel\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x36\n\x05\x66\x61\x63\x65s\x18\x02 \x01(\x0b\x32%.hedgehog.protocol.proto.FacesChannelH\x00\x12\x36\n\x05\x62lobs\x18\x03 \x01(\x0b\x32%.hedgehog.protocol.proto.BlobsChannelH\x00\x42\t\n\x07\x63hannel\";\n\x04\x46\x61\x63\x65\x12\t\n\x01x\x18\x01 \x01(\r\x12\t\n\x01y\x18\x02 \x01(\r\x12\r\n\x05width\x18\x03 \x01(\r\x12\x0e\n\x06height\x18\x04 \x01(\r\"<\n\x0c\x46\x61\x63\x65sFeature\x12,\n\x05\x66\x61\x63\x65s\x18\x01 \x03(\x0b\x32\x1d.hedgehog.protocol.proto.Face\"g\n\x04\x42lob\x12\t\n\x01x\x18\x01 \x01(\r\x12\t\n\x01y\x18\x02 \x01(\r\x12\r\n\x05width\x18\x03 \x01(\r\x12\x0e\n\x06height\x18\x04 \x01(\r\x12\n\n\x02\x63x\x18\x05 \x01(\r\x12\n\n\x02\x63y\x18\x06 \x01(\r\x12\x12\n\nconfidence\x18\x07 \x01(\x02\"<\n\x0c\x42lobsFeature\x12,\n\x05\x62lobs\x18\x01 \x03(\x0b\x32\x1d.hedgehog.protocol.proto.Blob\"\x84\x01\n\x07\x46\x65\x61ture\x12\x36\n\x05\x66\x61\x63\x65s\x18\x01 \x01(\x0b\x32%.hedgehog.protocol.proto.FacesFeatureH\x00\x12\x36\n\x05\x62lobs\x18\x02 \x01(\x0b\x32%.hedgehog.protocol.proto.BlobsFeatureH\x00\x42\t\n\x07\x66\x65\x61ture\"\"\n\x12VisionCameraAction\x12\x0c\n\x04open\x18\x01 \x01(\x08\"\x81\x01\n\x14VisionChannelMessage\x12\x35\n\x02op\x18\x01 \x01(\x0e\x32).hedgehog.protocol.proto.ChannelOperation\x12\x32\n\x08\x63hannels\x18\x02 \x03(\x0b\x32 .hedgehog.protocol.proto.Channel\"\x1a\n\x18VisionCaptureFrameAction\"6\n\x12VisionFrameMessage\x12\x11\n\thighlight\x18\x01 \x01(\t\x12\r\n\x05\x66rame\x18\x02 \x01(\x0c\"Z\n\x14VisionFeatureMessage\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\t\x12\x31\n\x07\x66\x65\x61ture\x18\x02 \x01(\x0b\x32 .hedgehog.protocol.proto.Feature*@\n\x10\x43hannelOperation\x12\x08\n\x04READ\x10\x00\x12\n\n\x06\x43REATE\x10\x01\x12\n\n\x06UPDATE\x10\x02\x12\n\n\x06\x44\x45LETE\x10\x03\x42*\n\x1f\x61t.pria.hedgehog.protocol.protoB\x07VisionPb\x06proto3')
)

_CHANNELOPERATION = _descriptor.EnumDescriptor(
  name='ChannelOperation',
  full_name='hedgehog.protocol.proto.ChannelOperation',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='READ', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CREATE', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UPDATE', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DELETE', index=3, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1048,
  serialized_end=1112,
)
_sym_db.RegisterEnumDescriptor(_CHANNELOPERATION)

ChannelOperation = enum_type_wrapper.EnumTypeWrapper(_CHANNELOPERATION)
READ = 0
CREATE = 1
UPDATE = 2
DELETE = 3



_FACESCHANNEL = _descriptor.Descriptor(
  name='FacesChannel',
  full_name='hedgehog.protocol.proto.FacesChannel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=65,
  serialized_end=79,
)


_BLOBSCHANNEL = _descriptor.Descriptor(
  name='BlobsChannel',
  full_name='hedgehog.protocol.proto.BlobsChannel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hsv_min', full_name='hedgehog.protocol.proto.BlobsChannel.hsv_min', index=0,
      number=1, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hsv_max', full_name='hedgehog.protocol.proto.BlobsChannel.hsv_max', index=1,
      number=2, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=81,
  serialized_end=129,
)


_CHANNEL = _descriptor.Descriptor(
  name='Channel',
  full_name='hedgehog.protocol.proto.Channel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='hedgehog.protocol.proto.Channel.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='faces', full_name='hedgehog.protocol.proto.Channel.faces', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blobs', full_name='hedgehog.protocol.proto.Channel.blobs', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='channel', full_name='hedgehog.protocol.proto.Channel.channel',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=132,
  serialized_end=277,
)


_FACE = _descriptor.Descriptor(
  name='Face',
  full_name='hedgehog.protocol.proto.Face',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='hedgehog.protocol.proto.Face.x', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='y', full_name='hedgehog.protocol.proto.Face.y', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='width', full_name='hedgehog.protocol.proto.Face.width', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='height', full_name='hedgehog.protocol.proto.Face.height', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=279,
  serialized_end=338,
)


_FACESFEATURE = _descriptor.Descriptor(
  name='FacesFeature',
  full_name='hedgehog.protocol.proto.FacesFeature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='faces', full_name='hedgehog.protocol.proto.FacesFeature.faces', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=340,
  serialized_end=400,
)


_BLOB = _descriptor.Descriptor(
  name='Blob',
  full_name='hedgehog.protocol.proto.Blob',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='hedgehog.protocol.proto.Blob.x', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='y', full_name='hedgehog.protocol.proto.Blob.y', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='width', full_name='hedgehog.protocol.proto.Blob.width', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='height', full_name='hedgehog.protocol.proto.Blob.height', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cx', full_name='hedgehog.protocol.proto.Blob.cx', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cy', full_name='hedgehog.protocol.proto.Blob.cy', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='confidence', full_name='hedgehog.protocol.proto.Blob.confidence', index=6,
      number=7, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=402,
  serialized_end=505,
)


_BLOBSFEATURE = _descriptor.Descriptor(
  name='BlobsFeature',
  full_name='hedgehog.protocol.proto.BlobsFeature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='blobs', full_name='hedgehog.protocol.proto.BlobsFeature.blobs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=507,
  serialized_end=567,
)


_FEATURE = _descriptor.Descriptor(
  name='Feature',
  full_name='hedgehog.protocol.proto.Feature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='faces', full_name='hedgehog.protocol.proto.Feature.faces', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blobs', full_name='hedgehog.protocol.proto.Feature.blobs', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='feature', full_name='hedgehog.protocol.proto.Feature.feature',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=570,
  serialized_end=702,
)


_VISIONCAMERAACTION = _descriptor.Descriptor(
  name='VisionCameraAction',
  full_name='hedgehog.protocol.proto.VisionCameraAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='open', full_name='hedgehog.protocol.proto.VisionCameraAction.open', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=704,
  serialized_end=738,
)


_VISIONCHANNELMESSAGE = _descriptor.Descriptor(
  name='VisionChannelMessage',
  full_name='hedgehog.protocol.proto.VisionChannelMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='op', full_name='hedgehog.protocol.proto.VisionChannelMessage.op', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='channels', full_name='hedgehog.protocol.proto.VisionChannelMessage.channels', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=741,
  serialized_end=870,
)


_VISIONCAPTUREFRAMEACTION = _descriptor.Descriptor(
  name='VisionCaptureFrameAction',
  full_name='hedgehog.protocol.proto.VisionCaptureFrameAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=872,
  serialized_end=898,
)


_VISIONFRAMEMESSAGE = _descriptor.Descriptor(
  name='VisionFrameMessage',
  full_name='hedgehog.protocol.proto.VisionFrameMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='highlight', full_name='hedgehog.protocol.proto.VisionFrameMessage.highlight', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='frame', full_name='hedgehog.protocol.proto.VisionFrameMessage.frame', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=900,
  serialized_end=954,
)


_VISIONFEATUREMESSAGE = _descriptor.Descriptor(
  name='VisionFeatureMessage',
  full_name='hedgehog.protocol.proto.VisionFeatureMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='channel', full_name='hedgehog.protocol.proto.VisionFeatureMessage.channel', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='feature', full_name='hedgehog.protocol.proto.VisionFeatureMessage.feature', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=956,
  serialized_end=1046,
)

_CHANNEL.fields_by_name['faces'].message_type = _FACESCHANNEL
_CHANNEL.fields_by_name['blobs'].message_type = _BLOBSCHANNEL
_CHANNEL.oneofs_by_name['channel'].fields.append(
  _CHANNEL.fields_by_name['faces'])
_CHANNEL.fields_by_name['faces'].containing_oneof = _CHANNEL.oneofs_by_name['channel']
_CHANNEL.oneofs_by_name['channel'].fields.append(
  _CHANNEL.fields_by_name['blobs'])
_CHANNEL.fields_by_name['blobs'].containing_oneof = _CHANNEL.oneofs_by_name['channel']
_FACESFEATURE.fields_by_name['faces'].message_type = _FACE
_BLOBSFEATURE.fields_by_name['blobs'].message_type = _BLOB
_FEATURE.fields_by_name['faces'].message_type = _FACESFEATURE
_FEATURE.fields_by_name['blobs'].message_type = _BLOBSFEATURE
_FEATURE.oneofs_by_name['feature'].fields.append(
  _FEATURE.fields_by_name['faces'])
_FEATURE.fields_by_name['faces'].containing_oneof = _FEATURE.oneofs_by_name['feature']
_FEATURE.oneofs_by_name['feature'].fields.append(
  _FEATURE.fields_by_name['blobs'])
_FEATURE.fields_by_name['blobs'].containing_oneof = _FEATURE.oneofs_by_name['feature']
_VISIONCHANNELMESSAGE.fields_by_name['op'].enum_type = _CHANNELOPERATION
_VISIONCHANNELMESSAGE.fields_by_name['channels'].message_type = _CHANNEL
_VISIONFEATUREMESSAGE.fields_by_name['feature'].message_type = _FEATURE
DESCRIPTOR.message_types_by_name['FacesChannel'] = _FACESCHANNEL
DESCRIPTOR.message_types_by_name['BlobsChannel'] = _BLOBSCHANNEL
DESCRIPTOR.message_types_by_name['Channel'] = _CHANNEL
DESCRIPTOR.message_types_by_name['Face'] = _FACE
DESCRIPTOR.message_types_by_name['FacesFeature'] = _FACESFEATURE
DESCRIPTOR.message_types_by_name['Blob'] = _BLOB
DESCRIPTOR.message_types_by_name['BlobsFeature'] = _BLOBSFEATURE
DESCRIPTOR.message_types_by_name['Feature'] = _FEATURE
DESCRIPTOR.message_types_by_name['VisionCameraAction'] = _VISIONCAMERAACTION
DESCRIPTOR.message_types_by_name['VisionChannelMessage'] = _VISIONCHANNELMESSAGE
DESCRIPTOR.message_types_by_name['VisionCaptureFrameAction'] = _VISIONCAPTUREFRAMEACTION
DESCRIPTOR.message_types_by_name['VisionFrameMessage'] = _VISIONFRAMEMESSAGE
DESCRIPTOR.message_types_by_name['VisionFeatureMessage'] = _VISIONFEATUREMESSAGE
DESCRIPTOR.enum_types_by_name['ChannelOperation'] = _CHANNELOPERATION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FacesChannel = _reflection.GeneratedProtocolMessageType('FacesChannel', (_message.Message,), dict(
  DESCRIPTOR = _FACESCHANNEL,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.FacesChannel)
  ))
_sym_db.RegisterMessage(FacesChannel)

BlobsChannel = _reflection.GeneratedProtocolMessageType('BlobsChannel', (_message.Message,), dict(
  DESCRIPTOR = _BLOBSCHANNEL,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.BlobsChannel)
  ))
_sym_db.RegisterMessage(BlobsChannel)

Channel = _reflection.GeneratedProtocolMessageType('Channel', (_message.Message,), dict(
  DESCRIPTOR = _CHANNEL,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.Channel)
  ))
_sym_db.RegisterMessage(Channel)

Face = _reflection.GeneratedProtocolMessageType('Face', (_message.Message,), dict(
  DESCRIPTOR = _FACE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.Face)
  ))
_sym_db.RegisterMessage(Face)

FacesFeature = _reflection.GeneratedProtocolMessageType('FacesFeature', (_message.Message,), dict(
  DESCRIPTOR = _FACESFEATURE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.FacesFeature)
  ))
_sym_db.RegisterMessage(FacesFeature)

Blob = _reflection.GeneratedProtocolMessageType('Blob', (_message.Message,), dict(
  DESCRIPTOR = _BLOB,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.Blob)
  ))
_sym_db.RegisterMessage(Blob)

BlobsFeature = _reflection.GeneratedProtocolMessageType('BlobsFeature', (_message.Message,), dict(
  DESCRIPTOR = _BLOBSFEATURE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.BlobsFeature)
  ))
_sym_db.RegisterMessage(BlobsFeature)

Feature = _reflection.GeneratedProtocolMessageType('Feature', (_message.Message,), dict(
  DESCRIPTOR = _FEATURE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.Feature)
  ))
_sym_db.RegisterMessage(Feature)

VisionCameraAction = _reflection.GeneratedProtocolMessageType('VisionCameraAction', (_message.Message,), dict(
  DESCRIPTOR = _VISIONCAMERAACTION,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.VisionCameraAction)
  ))
_sym_db.RegisterMessage(VisionCameraAction)

VisionChannelMessage = _reflection.GeneratedProtocolMessageType('VisionChannelMessage', (_message.Message,), dict(
  DESCRIPTOR = _VISIONCHANNELMESSAGE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.VisionChannelMessage)
  ))
_sym_db.RegisterMessage(VisionChannelMessage)

VisionCaptureFrameAction = _reflection.GeneratedProtocolMessageType('VisionCaptureFrameAction', (_message.Message,), dict(
  DESCRIPTOR = _VISIONCAPTUREFRAMEACTION,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.VisionCaptureFrameAction)
  ))
_sym_db.RegisterMessage(VisionCaptureFrameAction)

VisionFrameMessage = _reflection.GeneratedProtocolMessageType('VisionFrameMessage', (_message.Message,), dict(
  DESCRIPTOR = _VISIONFRAMEMESSAGE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.VisionFrameMessage)
  ))
_sym_db.RegisterMessage(VisionFrameMessage)

VisionFeatureMessage = _reflection.GeneratedProtocolMessageType('VisionFeatureMessage', (_message.Message,), dict(
  DESCRIPTOR = _VISIONFEATUREMESSAGE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.VisionFeatureMessage)
  ))
_sym_db.RegisterMessage(VisionFeatureMessage)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\037at.pria.hedgehog.protocol.protoB\007VisionP'))
# @@protoc_insertion_point(module_scope)
